@@ -21,9 +21,14 @@ except ImportError:
     from pysqlite2 import dbapi2 as sqlite3
 import urlparse
 
+# http://hg.mozilla.org/mozilla-central/file/b871dfb2186f/build/automation.py.in#l28
+_DEFAULT_PORTS = { 'http': '8888',
+                   'https': '4443',
+                   'ws': '9988',
+                   'wss': '4443' }
 
 class LocationError(Exception):
-    "Signifies an improperly formed location."
+    """Signifies an improperly formed location."""
 
     def __str__(self):
         s = "Bad location"
@@ -33,35 +38,35 @@ class LocationError(Exception):
 
 
 class MissingPrimaryLocationError(LocationError):
-    "No primary location defined in locations file."
+    """No primary location defined in locations file."""
 
     def __init__(self):
         LocationError.__init__(self, "missing primary location")
 
 
 class MultiplePrimaryLocationsError(LocationError):
-    "More than one primary location defined."
+    """More than one primary location defined."""
 
     def __init__(self):
         LocationError.__init__(self, "multiple primary locations")
 
 
 class DuplicateLocationError(LocationError):
-    "Same location defined twice."
+    """Same location defined twice."""
 
     def __init__(self, url):
         LocationError.__init__(self, "duplicate location: %s" % url)
 
 
 class BadPortLocationError(LocationError):
-    "Location has invalid port value."
+    """Location has invalid port value."""
 
     def __init__(self, given_port):
         LocationError.__init__(self, "bad value for port: %s" % given_port)
-        
+
 
 class LocationsSyntaxError(Exception):
-    "Signifies a syntax error on a particular line in server-locations.txt."
+    """Signifies a syntax error on a particular line in server-locations.txt."""
 
     def __init__(self, lineno, err=None):
         self.err = err
@@ -77,7 +82,7 @@ class LocationsSyntaxError(Exception):
 
 
 class Location(object):
-    "Represents a location line in server-locations.txt."
+    """Represents a location line in server-locations.txt."""
 
     attrs = ('scheme', 'host', 'port')
 
@@ -91,7 +96,7 @@ class Location(object):
             raise BadPortLocationError(self.port)
 
     def isEqual(self, location):
-        "compare scheme://host:port, but ignore options"
+        """compare scheme://host:port, but ignore options"""
         return len([i for i in self.attrs if getattr(self, i) == getattr(location, i)]) == len(self.attrs)
 
     __eq__ = isEqual
@@ -128,9 +133,7 @@ class ServerLocations(object):
             if self.hasPrimary:
                 raise MultiplePrimaryLocationsError()
             self.hasPrimary = True
-        for loc in self._locations:
-            if loc.isEqual(location):
-                raise DuplicateLocationError(location.url())
+
         self._locations.append(location)
         if self.add_callback and not suppress_callback:
             self.add_callback([location])
@@ -142,15 +145,14 @@ class ServerLocations(object):
 
     def read(self, filename, check_for_primary=True):
         """
-        Reads the file (in the format of server-locations.txt) and add all
-        valid locations to the self._locations array.
+        Reads the file and adds all valid locations to the ``self._locations`` array.
 
-        If check_for_primary is True, a MissingPrimaryLocationError
-        exception is raised if no primary is found.
+        :param filename: in the format of server-locations.txt_
+        :param check_for_primary: if True, a ``MissingPrimaryLocationError`` exception is raised if no primary is found
 
-        This format:
-        http://mxr.mozilla.org/mozilla-central/source/build/pgo/server-locations.txt
-        The only exception is that the port, if not defined, defaults to 80.
+        .. _server-locations.txt: http://mxr.mozilla.org/mozilla-central/source/build/pgo/server-locations.txt
+
+        The only exception is that the port, if not defined, defaults to 80 or 443.
 
         FIXME: Shouldn't this default to the protocol-appropriate port?  Is
         there any reason to have defaults at all?
@@ -185,7 +187,7 @@ class ServerLocations(object):
                 host, port = netloc.rsplit(':', 1)
             except ValueError:
                 host = netloc
-                port = '80'
+                port = _DEFAULT_PORTS.get(scheme, '80')
 
             try:
                 location = Location(scheme, host, port, options)
@@ -205,6 +207,8 @@ class ServerLocations(object):
 
 
 class Permissions(object):
+    """Allows handling of permissions for ``mozprofile``"""
+
     _num_permissions = 0
 
     def __init__(self, profileDir, locations=None):
@@ -229,6 +233,7 @@ class Permissions(object):
         # Open database and create table
         permDB = sqlite3.connect(os.path.join(self._profileDir, "permissions.sqlite"))
         cursor = permDB.cursor();
+        cursor.execute("PRAGMA schema_version = 3;")
         # SQL copied from
         # http://mxr.mozilla.org/mozilla-central/source/extensions/cookie/nsPermissionManager.cpp
         cursor.execute("""CREATE TABLE IF NOT EXISTS moz_hosts (
@@ -256,7 +261,7 @@ class Permissions(object):
         permDB.commit()
         cursor.close()
 
-    def network_prefs(self, proxy=False):
+    def network_prefs(self, proxy=None):
         """
         take known locations and generate preferences to handle permissions and proxy
         returns a tuple of prefs, user_prefs
@@ -268,39 +273,43 @@ class Permissions(object):
         for (i, l) in itertools.izip(itertools.count(1), privileged):
             prefs.append(("capability.principal.codebase.p%s.granted" % i, "UniversalXPConnect"))
 
-            # TODO: do we need the port?
-            prefs.append(("capability.principal.codebase.p%s.id" % i, l.scheme + "://" + l.host))
+            prefs.append(("capability.principal.codebase.p%s.id" % i, "%s://%s:%s" %
+                        (l.scheme, l.host, l.port)))
             prefs.append(("capability.principal.codebase.p%s.subjectName" % i, ""))
 
         if proxy:
-            user_prefs = self.pac_prefs()
+            user_prefs = self.pac_prefs(proxy)
         else:
             user_prefs = []
 
         return prefs, user_prefs
 
-    def pac_prefs(self):
+    def pac_prefs(self, user_proxy=None):
         """
         return preferences for Proxy Auto Config. originally taken from
         http://mxr.mozilla.org/mozilla-central/source/build/automation.py.in
         """
-
-        prefs = []
+        proxy = _DEFAULT_PORTS
 
         # We need to proxy every server but the primary one.
         origins = ["'%s'" % l.url()
-                   for l in self._locations
-                   if "primary" not in l.options]
+                   for l in self._locations]
         origins = ", ".join(origins)
+        proxy["origins"] = origins
 
-        # TODO: this is not a reliable way to determine the Proxy host
         for l in self._locations:
             if "primary" in l.options:
-                webServer = l.host
-                httpPort  = l.port
-                sslPort   = 443
+                proxy["remote"] = l.host
+                proxy[l.scheme] = l.port
+
+        # overwrite defaults with user specified proxy
+        if isinstance(user_proxy, dict):
+            proxy.update(user_proxy)
 
         # TODO: this should live in a template!
+        # TODO: So changing the 5th line of the regex below from (\\\\\\\\d+)
+        # to (\\\\d+) makes this code work. Not sure why there would be this
+        # difference between automation.py.in and this file.
         pacURL = """data:text/plain,
 function FindProxyForURL(url, host)
 {
@@ -309,7 +318,7 @@ function FindProxyForURL(url, host)
                          '://' +
                          '(?:[^/@]*@)?' +
                          '(.*?)' +
-                         '(?::(\\\\\\\\d+))?/');
+                         '(?::(\\\\d+))?/');
   var matches = regex.exec(url);
   if (!matches)
     return 'DIRECT';
@@ -330,17 +339,15 @@ function FindProxyForURL(url, host)
   var origin = matches[1] + '://' + matches[2] + ':' + matches[3];
   if (origins.indexOf(origin) < 0)
     return 'DIRECT';
-  if (isHttp)
-    return 'PROXY %(remote)s:%(httpport)s';
-  if (isHttps || isWebSocket || isWebSocketSSL)
-    return 'PROXY %(remote)s:%(sslport)s';
+  if (isHttp) return 'PROXY %(remote)s:%(http)s';
+  if (isHttps) return 'PROXY %(remote)s:%(https)s';
+  if (isWebSocket) return 'PROXY %(remote)s:%(ws)s';
+  if (isWebSocketSSL) return 'PROXY %(remote)s:%(wss)s';
   return 'DIRECT';
-}""" % { "origins": origins,
-         "remote":  webServer,
-         "httpport":httpPort,
-         "sslport": sslPort }
+}""" % proxy
         pacURL = "".join(pacURL.splitlines())
 
+        prefs = []
         prefs.append(("network.proxy.type", 2))
         prefs.append(("network.proxy.autoconfig_url", pacURL))
 
@@ -349,8 +356,12 @@ function FindProxyForURL(url, host)
     def clean_db(self):
         """Removed permissions added by mozprofile."""
 
+        sqlite_file = os.path.join(self._profileDir, "permissions.sqlite")
+        if not os.path.exists(sqlite_file):
+            return
+
         # Open database and create table
-        permDB = sqlite3.connect(os.path.join(self._profileDir, "permissions.sqlite"))
+        permDB = sqlite3.connect(sqlite_file)
         cursor = permDB.cursor();
 
         # TODO: only delete values that we add, this would require sending in the full permissions object
